package sun.security.acl;

import java.security.acl.Group;
import java.util.Enumeration;
import java.security.acl.NotOwnerException;
import java.security.acl.Permission;
import java.util.Vector;
import java.security.acl.AclEntry;
import java.security.Principal;
import java.util.Hashtable;
import java.security.acl.Acl;

public class AclImpl extends OwnerImpl implements Acl
{
    private Hashtable<Principal, AclEntry> allowedUsersTable;
    private Hashtable<Principal, AclEntry> allowedGroupsTable;
    private Hashtable<Principal, AclEntry> deniedUsersTable;
    private Hashtable<Principal, AclEntry> deniedGroupsTable;
    private String aclName;
    private Vector<Permission> zeroSet;
    
    public AclImpl(Principal principal, String s) {
        super(principal);
        this.allowedUsersTable = new Hashtable(23);
        this.allowedGroupsTable = new Hashtable(23);
        this.deniedUsersTable = new Hashtable(23);
        this.deniedGroupsTable = new Hashtable(23);
        this.aclName = null;
        this.zeroSet = new Vector(1, 1);
        try {
            this.setName(principal, s);
        }
        catch (Exception ex) {}
    }
    
    public void setName(Principal principal, String aclName) throws NotOwnerException {
        if (!this.isOwner(principal)) {
            throw new NotOwnerException();
        }
        this.aclName = aclName;
        return;
    }
    
    public String getName() {
        return this.aclName;
    }
    
    public synchronized boolean addEntry(Principal principal, AclEntry value) throws NotOwnerException {
        if (!this.isOwner(principal)) {
            throw new NotOwnerException();
        }
        Hashtable<Principal, AclEntry> table = this.findTable(value);
        Principal principal2 = value.getPrincipal();
        if (table.get(principal2) != null) {
            return false;
        }
        table.put(principal2, value);
        return true;
    }
    
    public synchronized boolean removeEntry(Principal principal, AclEntry aclEntry) throws NotOwnerException {
        if (!this.isOwner(principal)) {
            throw new NotOwnerException();
        }
        return (AclEntry)this.findTable(aclEntry).remove(aclEntry.getPrincipal()) != null;
    }
    
    public synchronized Enumeration<Permission> getPermissions(Principal principal) {
        return this.subtract(union(this.subtract(this.getIndividualPositive(principal), this.getIndividualNegative(principal)), this.subtract(this.subtract(this.getGroupPositive(principal), this.getGroupNegative(principal)), this.subtract(this.getIndividualNegative(principal), this.getIndividualPositive(principal)))), union(this.subtract(this.getIndividualNegative(principal), this.getIndividualPositive(principal)), this.subtract(this.subtract(this.getGroupNegative(principal), this.getGroupPositive(principal)), this.subtract(this.getIndividualPositive(principal), this.getIndividualNegative(principal)))));
    }
    
    public boolean checkPermission(Principal principal, Permission permission) {
        Enumeration<Permission> permissions = this.getPermissions(principal);
        while (permissions.hasMoreElements()) {
            if (((Permission)permissions.nextElement()).equals(permission)) {
                return true;
            }
            continue;
        }
        return false;
    }
    
    public synchronized Enumeration<AclEntry> entries() {
        return new AclEnumerator(this, this.allowedUsersTable, this.allowedGroupsTable, this.deniedUsersTable, this.deniedGroupsTable);
    }
    
    public String toString() {
        StringBuffer sb = new StringBuffer();
        Enumeration<AclEntry> entries = this.entries();
        while (entries.hasMoreElements()) {
            sb.append(((AclEntry)entries.nextElement()).toString().trim());
            sb.append("\n");
        }
        return sb.toString();
    }
    
    private Hashtable<Principal, AclEntry> findTable(AclEntry aclEntry) {
        Hashtable<Object, Object> hashtable;
        if ((aclEntry.getPrincipal() instanceof Group)) {
            if (aclEntry.isNegative()) {
                hashtable = this.deniedGroupsTable;
            }
            else {
                hashtable = this.allowedGroupsTable;
            }
        }
        else if (aclEntry.isNegative()) {
            hashtable = this.deniedUsersTable;
        }
        else {
            hashtable = this.allowedUsersTable;
        }
        return hashtable;
    }
    
    private static Enumeration<Permission> union(Enumeration<Permission> enumeration, Enumeration<Permission> enumeration2) {
        Vector vector = new Vector(20, 20);
        while (enumeration.hasMoreElements()) {
            vector.addElement(enumeration.nextElement());
        }
        while (enumeration2.hasMoreElements()) {
            Permission permission = (Permission)enumeration2.nextElement();
            if (!vector.contains(permission)) {
                vector.addElement(permission);
            }
            else {
                continue;
            }
        }
        return vector.elements();
    }
    
    private Enumeration<Permission> subtract(Enumeration<Permission> enumeration, Enumeration<Permission> enumeration2) {
        Vector vector = new Vector(20, 20);
        while (enumeration.hasMoreElements()) {
            vector.addElement(enumeration.nextElement());
        }
        while (enumeration2.hasMoreElements()) {
            Permission permission = (Permission)enumeration2.nextElement();
            if (vector.contains(permission)) {
                vector.removeElement(permission);
            }
            else {
                continue;
            }
        }
        return vector.elements();
    }
    
    private Enumeration<Permission> getGroupPositive(Principal principal) {
        Enumeration<Permission> enumeration = this.zeroSet.elements();
        Enumeration<Principal> keys = this.allowedGroupsTable.keys();
        while (keys.hasMoreElements()) {
            Group key = (Group)keys.nextElement();
            if (key.isMember(principal)) {
                enumeration = union(((AclEntry)this.allowedGroupsTable.get(key)).permissions(), enumeration);
            }
            else {
                continue;
            }
        }
        return enumeration;
    }
    
    private Enumeration<Permission> getGroupNegative(Principal principal) {
        Enumeration<Permission> enumeration = this.zeroSet.elements();
        Enumeration<Principal> keys = this.deniedGroupsTable.keys();
        while (keys.hasMoreElements()) {
            Group key = (Group)keys.nextElement();
            if (key.isMember(principal)) {
                enumeration = union(((AclEntry)this.deniedGroupsTable.get(key)).permissions(), enumeration);
            }
            else {
                continue;
            }
        }
        return enumeration;
    }
    
    private Enumeration<Permission> getIndividualPositive(Principal key) {
        Enumeration<Permission> enumeration = this.zeroSet.elements();
        AclEntry aclEntry = (AclEntry)this.allowedUsersTable.get(key);
        if (aclEntry != null) {
            enumeration = aclEntry.permissions();
            return enumeration;
        }
        return enumeration;
    }
    
    private Enumeration<Permission> getIndividualNegative(Principal key) {
        Enumeration<Permission> enumeration = this.zeroSet.elements();
        AclEntry aclEntry = (AclEntry)this.deniedUsersTable.get(key);
        if (aclEntry != null) {
            enumeration = aclEntry.permissions();
            return enumeration;
        }
        return enumeration;
    }
}
